# Generated by Django 5.0.1 on 2024-01-27 10:58

import django.db.models.deletion
import django.db.models.functions.datetime
import markdownx.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DjDynamicTemplateCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('app', models.CharField(help_text="Select the Django app where a directory will be created within the 'templates' folder. This helps organize templates under the selected app.", max_length=100, verbose_name='App')),
                ('name', models.CharField(help_text="Enter a unique name for the directory that will be created. This name will be used as the directory name within the 'templates' folder for the selected app.", max_length=100, verbose_name='Category Name')),
                ('description', models.TextField(blank=True, help_text='Provide a brief description of the category. This description is optional and can help document the purpose or content of the templates within the created directory', null=True, verbose_name='Description')),
                ('created_at', models.DateTimeField(db_default=django.db.models.functions.datetime.Now(), editable=False, help_text='The date and time when this template category was created. This field is automatically set to the current date and time when the record is first created.', verbose_name='Created at')),
                ('last_updated_at', models.DateTimeField(auto_now=True, help_text='The date and time when this template category was last updated. This field is automatically updated every time the record is modified.', verbose_name='Last Updated at')),
                ('created_by', models.ForeignKey(blank=True, editable=False, help_text='The user who created this template category. This field is automatically populated with the user who initially created the record.', null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='Created by')),
                ('last_updated_by', models.ForeignKey(blank=True, editable=False, help_text='The user who last updated this template category. This field is automatically populated with the user making the modification.', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='dj_dynamic_template_category_last_updated_by', to=settings.AUTH_USER_MODEL, verbose_name='Last Updated by')),
            ],
            options={
                'verbose_name': 'Dj Dynamic Template Category',
                'verbose_name_plural': 'Dj Dynamic Template Categories',
                'db_table': 'dj_dynamic_template_category',
                'ordering': ('app',),
                'permissions': (('can_create_directory', 'Can Create Directory'), ('can_delete_directory', 'Can Delete Directory'), ('can_view_files_in_directory', 'Can View Files in Directory'), ('can_view_directory_status', 'Can View Directory Status')),
                'managed': True,
                'unique_together': {('app', 'name')},
            },
        ),
        migrations.CreateModel(
            name='DjDynamicTemplate',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('template_name', models.CharField(help_text='Enter a name for the template. This name will be treated as the template file name. A template file(.html) with this name will be created.', max_length=100, verbose_name='Template Name')),
                ('content', markdownx.models.MarkdownxField(blank=True, help_text='Insert the content of the template. This field allows you to define the content of your template, which can include HTML and inline CSS', null=True, verbose_name='Content')),
                ('template_is_active', models.BooleanField(default=True, help_text="This field determines the current status of the template. If set to 'True', the template is considered active. If set to 'False', it is considered inactive. Inactive templates will not clone the templates", verbose_name='Template is Active')),
                ('created_at', models.DateTimeField(db_default=django.db.models.functions.datetime.Now(), editable=False, help_text='The date and time when this template was created. This field is automatically set to the current date and time when the record is first created.', verbose_name='Created at')),
                ('created_by', models.ForeignKey(blank=True, editable=False, help_text='The user who created this template. This field is automatically populated with the user who initially created the record.', null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='Created by')),
                ('revision_of', models.ForeignKey(blank=True, editable=False, help_text='This field is used when editing a template. Create a new record to represent the updated version of the template and link it to the previous version using this field. It indicates the previous template revision that this record is based on.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='dj_dynamic_templates.djdynamictemplate', verbose_name='Previous Template Revision')),
                ('category', models.ForeignKey(help_text='Select the category where you want to save the template. This categorization helps organize and manage your templates efficiently.', on_delete=django.db.models.deletion.CASCADE, to='dj_dynamic_templates.djdynamictemplatecategory', verbose_name='Category')),
            ],
            options={
                'verbose_name': 'Dj Dynamic Template',
                'verbose_name_plural': 'Dj Dynamic Templates',
                'db_table': 'dj_dynamic_template',
                'ordering': ('-created_at',),
                'permissions': (('can_view_inactive_templates', 'Can view the Inactive Templates'), ('can_create_file', 'Can Create File'), ('can_delete_file', 'Can Delete File'), ('can_view_file_status', 'Can View File Status')),
                'managed': True,
            },
        ),
        migrations.AddConstraint(
            model_name='djdynamictemplate',
            constraint=models.UniqueConstraint(condition=models.Q(('template_is_active', True)), fields=('template_name', 'category'), name='unique_category_template', violation_error_code='DUPLICATE_TEMPLATE_IN_APP_CATEGORY', violation_error_message='A template with the same name already exists in this category. Please choose a different name or deactivate the existing template.'),
        ),
    ]



from django.conf import settings
from django.apps import apps
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _
import os
import shutil
from django.db import models
from django.db.models.functions import Now


class DjDynamicTemplateCategory(models.Model):

    app = models.CharField(
        verbose_name=_('App'), max_length=100,
        help_text=_("Select the Django app where a directory will be created within the 'templates' folder. "
                    "This helps organize templates under the selected app.")
    )
    name = models.CharField(
        verbose_name=_('Category Name'), max_length=100,
        help_text=_("Enter a unique name for the directory that will be created. "
                    "This name will be used as the directory name within the 'templates' folder for the selected app.")
    )

    description = models.TextField(
        verbose_name=_('Description'), null=True, blank=True,
        help_text=_("Provide a brief description of the category. "
                    "This description is optional and can help document the purpose or content of the templates within the created directory")
    )

    created_at = models.DateTimeField(
        verbose_name=_('Created at'), db_default=Now(), editable=False,
        help_text=_("The date and time when this template category was created. "
                    "This field is automatically set to the current date and time when the record is first created.")
    )
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, null=True, blank=True, editable=False, verbose_name=_('Created by'),
        help_text=_("The user who created this template category. "
                    "This field is automatically populated with the user who initially created the record.")
    )

    last_updated_at = models.DateTimeField(
        verbose_name=_('Last Updated at'), auto_now=True, editable=False,
        help_text=_("The date and time when this template category was last updated. "
                    "This field is automatically updated every time the record is modified.")
    )

    last_updated_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, null=True, blank=True, editable=False,
        related_name='dj_dynamic_template_category_last_updated_by', verbose_name=_('Last Updated by'),
        help_text=_("The user who last updated this template category. "
                    "This field is automatically populated with the user making the modification.")
    )

    def __str__(self):
        return f'{self.app} - {self.name}'

    @cached_property
    def directory_path(self):
        return os.path.join(settings.BASE_DIR, self.app, 'templates', self.name)

    @cached_property
    def is_directory_exists(self):
        return os.path.exists(self.directory_path)

    @cached_property
    def files_in_dir(self):
        if self.is_directory_exists:
            return os.listdir(self.directory_path)
        else:
            return list()

    def make_directory(self):
        if not self.is_directory_exists:
            os.makedirs(self.directory_path, exist_ok=False)
            return True
        return False

    def remove_directory(self):
        if self.is_directory_exists:
            shutil.rmtree(self.directory_path, ignore_errors=True)
            return True
        else:
            return False


    class Meta:
        managed = apps.is_installed("dj_dynamic_templates")
        ordering = ('app', )
        unique_together = ('app', 'name')
        permissions = (
            ('can_create_directory', 'Can Create Directory'),
            ('can_delete_directory', 'Can Delete Directory'),
            ('can_view_files_in_directory', 'Can View Files in Directory'),
            ('can_view_directory_status', 'Can View Directory Status')
        )
        db_table = 'dj_dynamic_template_category'
        verbose_name = _(db_table.replace("_", " ").title())
        verbose_name_plural = verbose_name.replace("Category", "Categories")


class DjDynamicTemplate(models.Model):

    category = models.ForeignKey(
        DjDynamicTemplateCategory, on_delete=models.CASCADE, verbose_name=_('Category'),
        help_text=_("Select the category where you want to save the template. "
                    "This categorization helps organize and manage your templates efficiently.")
    )
    template_name = models.CharField(
        verbose_name=_('Template Name'), max_length=100,
        help_text=_("Enter a name for the template. This name will be treated as the template file name. "
                    "A template file(.html) with this name will be created.")
    )
    try:
        from markdownx.models import MarkdownxField
        content = MarkdownxField(
            verbose_name=_('Content'), null=True, blank=True,
            help_text=_("Insert the content of the template. "
                        "This field allows you to define the content of your template, which can include HTML and inline CSS")
        )
    except ModuleNotFoundError:
        content = models.TextField(
            verbose_name=_('Content'), null=True, blank=True,
            help_text=_("Insert the content of the template. "
                        "This field allows you to define the content of your template, which can include HTML and inline CSS")
        )

    template_is_active = models.BooleanField(
        verbose_name=_("Template is Active"), default=True,
        help_text=_("This field determines the current status of the template. "
                    "If set to 'True', the template is considered active. "
                    "If set to 'False', it is considered inactive. Inactive templates will not clone the templates")
    )

    revision_of = models.ForeignKey(
        'self', on_delete=models.SET_NULL, null=True, blank=True, editable=False, verbose_name=_("Previous Template Revision"),
        help_text=_("This field is used when editing a template. "
                    "Create a new record to represent the updated version of the template and link it to the previous version using this field. "
                    "It indicates the previous template revision that this record is based on.")
    )

    created_at = models.DateTimeField(
        verbose_name=_('Created at'), db_default=Now(), editable=False,
        help_text=_("The date and time when this template was created. "
                    "This field is automatically set to the current date and time when the record is first created.")
    )

    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, null=True, blank=True, editable=False, verbose_name=_('Created by'),
        help_text=_("The user who created this template. "
                    "This field is automatically populated with the user who initially created the record.")
    )

    @cached_property
    def file_path(self):
        return self.category.directory_path + f'/{self.template_name}.html'

    @cached_property
    def is_file_exist(self):
        return os.path.exists(self.file_path)

    def create_file(self):
        if self.category.is_directory_exists:
            file = open(f'{self.file_path}', 'w')
            file.write(self.content)
            file.close()
            return True
        else:
            return False

    def delete_file(self):
        if self.is_file_exist:
            os.remove(self.file_path)
            return True
        else:
            return False

    def __str__(self):
        return f'{self.category.app} - {self.category.name} - {self.template_name}'


    class Meta:
        managed = apps.is_installed("dj_dynamic_templates")
        ordering = ('-created_at', )
        constraints = [
            models.UniqueConstraint(
                fields=('template_name', 'category'),
                condition=models.Q(template_is_active=True),
                name='unique_category_template',
                violation_error_code='DUPLICATE_TEMPLATE_IN_APP_CATEGORY',
                violation_error_message=_('A template with the same name already exists in this category. '
                                          'Please choose a different name or deactivate the existing template.')
            )
        ]
        permissions = (
            ('can_view_inactive_templates', 'Can view the Inactive Templates'),
            ('can_create_file', 'Can Create File'),
            ('can_delete_file', 'Can Delete File'),
            ('can_view_file_status', 'Can View File Status'),
        )
        db_table = 'dj_dynamic_template'
        verbose_name = _(db_table.replace("_", " ").title())
        verbose_name_plural = verbose_name.replace("Template", "Templates")



from django.conf import settings
import os

from django import forms
from .models import *


DJANGO_APPS = []

for app in os.listdir(settings.BASE_DIR):
    if os.path.isdir(os.path.join(settings.BASE_DIR, app)) and apps.is_installed(app):
        DJANGO_APPS.append((app, app.replace("_", " ").title()))

class DjDynamicTemplateCategoryForm(forms.ModelForm):

    app = forms.ChoiceField(choices=DJANGO_APPS, widget=forms.Select())


    class Meta:
        model = DjDynamicTemplateCategory
        fields = '__all__'


import os
from django.utils.html import format_html
from django.shortcuts import redirect
from django.urls import reverse, path, include
from django.contrib import admin, messages
from django.http import HttpResponseRedirect
from django.contrib.auth.admin import UserAdmin
from .forms import *
from .views import template_view

try:
    from markdownx.admin import MarkdownxModelAdmin
except ModuleNotFoundError:
    class MarkdownxModelAdmin(admin.ModelAdmin):
        pass


@admin.register(DjDynamicTemplateCategory)
class DjDynamicTemplateCategoryAdmin(admin.ModelAdmin):

    list_display = ['id', 'app', 'name', 'last_updated_at', 'last_updated_by']
    list_per_page = 20
    list_filter = ['app', 'created_by', 'last_updated_by', 'last_updated_at']
    list_select_related = True
    list_display_links = ['id', 'app', 'name']
    search_fields = ['name']
    search_help_text = 'Search with Category Name'
    date_hierarchy = 'created_at'

    change_form_template = 'category_change_form.html'
    form = DjDynamicTemplateCategoryForm
    actions = ['create_directory', 'delete_directory']
    readonly_fields = ['created_by', 'created_at', 'last_updated_at', 'last_updated_by']

    fieldsets = [
        ('Basic Information', {
            'fields': ['app', 'name', 'description'],
            'classes': ['wide']
        }),
        ('Creation Details', {
            'fields': ['created_at', 'created_by'],
            'classes': ['collapse', 'wide']
        }),
        ('Update Details', {
            'fields': ['last_updated_at', 'last_updated_by'],
            'classes': ['collapse', 'wide']
        }),
    ]

    @staticmethod
    def is_directory_exists(obj):
        if obj.is_directory_exists:
            return format_html("""<img src="/static/admin/img/icon-yes.svg" alt="True">""")
        else:
            return format_html("""<img src="/static/admin/img/icon-no.svg" alt="True">""")

    def get_list_display(self, request):
        fields = self.list_display.copy()
        if request.user.has_perm('dj_dynamic_templates.can_view_directory_status'):
            fields.append('is_directory_exists')
        return fields

    def get_fieldsets(self, request, obj=None):
        if obj is None:
            return [self.fieldsets[0]]
        fieldsets = self.fieldsets.copy()
        if request.user.has_perm('dj_dynamic_templates.can_view_directory_status'):
            fieldsets.append(('Directory Details', {'fields': ['is_directory_exists'], 'classes': ['wide']}))
        if request.user.has_perm('dj_dynamic_templates.can_view_files_in_directory'):
            if fieldsets[-1][0] == 'Directory Details':
                fieldsets[-1][1]['fields'].append('files_in_directory')
            else:
                fieldsets.append(('Directory Details', {'fields': ['files_in_directory'], 'classes': ['wide']}))
        return fieldsets

    def get_actions(self, request):
        actions = super(DjDynamicTemplateCategoryAdmin, self).get_actions(request)
        if not request.user.has_perm('dj_dynamic_templates.can_create_directory'):
            del actions['create_directory']
        if not request.user.has_perm('dj_dynamic_templates.can_delete_directory'):
            del actions['delete_directory']
        return actions

    def get_readonly_fields(self, request, obj=None):
        fields = self.readonly_fields.copy()
        if obj:
            if request.user.has_perm('dj_dynamic_templates.can_view_directory_status'):
                fields.append('is_directory_exists')
            if request.user.has_perm('dj_dynamic_templates.can_view_files_in_directory'):
                fields.append('files_in_directory')
            return fields
        else:
            return []

    @staticmethod
    def files_in_directory(obj):
        return format_html("<ol>" + "".join(['<li>{file_name}</li>'.format(file_name=file) for file in obj.files_in_dir]) + "</ol>")

    @admin.action(description='Generate Category Folders for selected Records')
    def create_directory(self, request, queryset):
        for obj in queryset:
            if obj.make_directory():
                self.message_user(request, f"Directory '{obj.name}' has been successfully created in the 'templates' directory of the '{obj.app}' app.", messages.SUCCESS)
            else:
                self.message_user(request, f"Directory '{obj.name}' already exists in the 'templates' directory of the '{obj.app}' app.", messages.ERROR)

    @admin.action(description='Delete Directory for selected Records')
    def delete_directory(self, request, queryset):
        for obj in queryset:
            if obj.remove_directory():
                self.message_user(request, f"Directory '{obj.name}' has been successfully deleted in the 'templates' directory of the '{obj.app}' app.", messages.SUCCESS)
            else:
                self.message_user(request, f"Directory '{obj.name}' does not exist in the 'templates' directory of the '{obj.app}' app.", messages.ERROR)

    def changeform_view(self, request, object_id=None, form_url="", extra_context=None):
        if '_sync_dir' in request.POST:
            self.create_directory(request, [self.get_object(request, object_id)])
            return redirect('.')
        elif '_rmdir' in request.POST:
            self.delete_directory(request, [self.get_object(request, object_id)])
            return redirect('.')
        return super().changeform_view(request, object_id, form_url, extra_context)

    def save_model(self, request, obj, form, change):
        if change:
            obj.last_updated_by = request.user
            old_obj = self.model.objects.get(pk=obj.pk)
        else:
            obj.created_by = obj.last_updated_by = request.user
            if '_make_dir' in request.POST:
                self.create_directory(request, [obj])
        obj.save()
        if change and old_obj.directory_path != obj.directory_path and '_make_dir' in request.POST:
            try:
                os.rename(old_obj.directory_path, obj.directory_path)
                self.message_user(request, f"Directory path updated: Changed from '{old_obj.directory_path}' to '{obj.directory_path}'.", messages.SUCCESS)
            except FileExistsError:
                self.message_user(request, f"Directory '{obj.name}' already exists in the 'templates' directory of the '{obj.app}' app.", messages.ERROR)
            except FileNotFoundError:
                if old_obj.app != obj.app:
                    self.create_directory(request, [obj])
                else:
                    self.message_user(request, f"Directory '{old_obj.name}' does not exist in the 'templates' directory of the '{old_obj.app}' app.", messages.ERROR)

    def delete_model(self, request, obj):
        obj.remove_directory()
        obj.delete()


@admin.register(DjDynamicTemplate)
class DjDynamicTemplateAdmin(MarkdownxModelAdmin):
    list_display = ['category', 'template_name', 'created_at', 'created_by']
    list_select_related = True
    list_display_links = ['category', 'template_name', 'id']
    list_filter = ['category', 'created_by', 'created_at']
    readonly_fields = ['created_at', 'created_by']
    exclude = ['template_is_active']
    actions = ['sync_templates', 'delete_templates']
    change_form_template = 'template_change_form.html'

    def get_actions(self, request):
        actions = super(DjDynamicTemplateAdmin, self).get_actions(request)
        if not request.user.has_perm('dj_dynamic_templates.can_create_file'):
            del actions['sync_templates']
        if not request.user.has_perm('dj_dynamic_templates.can_delete_file'):
            del actions['delete_templates']
        return actions

    def get_exclude(self, request, obj=None):
        if not request.user.has_perm('dj_dynamic_templates.can_view_inactive_templates'):
            return self.exclude + ['revision_of']
        return super().get_exclude(request, obj)

    def render_change_form(self, request, context, add=False, change=False, form_url="", obj=None):
        if (change and obj) or add:
            context['template_is_active'] = obj.template_is_active if not add else True
        return super().render_change_form(request, context, add, change)

    def get_queryset(self, request):
        queryset = super(DjDynamicTemplateAdmin, self).get_queryset(request)
        if not request.user.has_perm('dj_dynamic_templates.can_view_inactive_templates'):
            queryset = queryset.exclude(template_is_active=False)
        return queryset

    def get_list_filter(self, request):
        list_filter = self.list_filter.copy()
        if request.user.has_perm('dj_dynamic_templates.can_view_inactive_templates'):
            list_filter += ['template_is_active']
        return list_filter

    def get_list_display(self, request):
        list_display = self.list_display.copy()
        if request.user.has_perm('dj_dynamic_templates.can_view_inactive_templates'):
            list_display += ['template_is_active']
            list_display.insert(0, 'id')
        if request.user.has_perm('dj_dynamic_templates.can_view_file_status'):
            list_display.append('template_status')
        return list_display

    def get_readonly_fields(self, request, obj=None):
        if obj:
            fields = self.readonly_fields + ['revision_of']
            if obj.template_is_active:
                if request.user.has_perm('dj_dynamic_templates.can_view_file_status'):
                    fields.append('template_status')
            return fields
        else:
            return []

    def save_model(self, request, obj, form, change):
        old_obj = None
        if not change:
            obj.created_by = request.user
            if '_make_template' in request.POST:
                self.sync_templates(request, [obj])
        elif '_make_template' in request.POST:
            old_obj = self.model.objects.get(pk=obj.pk)
            old_obj.template_is_active = False
            old_obj.save()
            old_obj.delete_file()
            obj.created_by = request.user
            obj.revision_of = old_obj
            obj.pk = None
            self.sync_templates(request, [obj])
        obj.save()

    @admin.action(description='Sync Templates to File for selected records')
    def sync_templates(self, request, queryset):
        for obj in queryset:
            if not obj.template_is_active:
                self.message_user(request, f"'{obj.template_name}' Template is Inactive. So, Unable to sync")
            elif obj.create_file():
                self.message_user(request, f"Successfully synced template '{obj.template_name}' into '{obj.category.name} directory of App '{obj.category.app}' template's directory", messages.SUCCESS)
            else:
                if obj.category.make_directory():
                    self.message_user(request, f"Directory does not exist so, '{obj.name}' directory has been created in the 'templates' directory of the '{obj.app}' app.", messages.WARNING)
                    obj.create_file()
                    self.message_user(request, f"Successfully synced template '{obj.template_name}' into '{obj.category.name} directory of App {obj.category.app}' template's directory", messages.SUCCESS)

    @admin.action(description='Delete Template Files for selected records')
    def delete_templates(self, request, queryset):
        for obj in queryset:
            if obj.delete_file():
                self.message_user(request, f"Successfully deleted template '{obj.template_name}' in '{obj.category.name} directory of App {obj.category.app}' template's directory", messages.SUCCESS)
            else:
                self.message_user(request, f"Template File does not exist for template {obj.template_name} to delete it...!", messages.ERROR)

    def has_change_permission(self, request, obj=None):
        return super(DjDynamicTemplateAdmin, self).has_change_permission(request, obj) and (obj.template_is_active if obj else True)

    def changeform_view(self, request, object_id=None, form_url="", extra_context=None):
        obj = self.get_object(request, object_id)
        if object_id:
            extra_context = extra_context or {}
            extra_context['show_save_and_continue'] = obj.template_is_active
            extra_context['show_save'] = obj.template_is_active
            extra_context['show_save_and_add_another'] = obj.template_is_active
        if '_sync_template' in request.POST:
            self.sync_templates(request, [obj])
            return redirect('.')
        elif '_remove_template' in request.POST:
            self.delete_templates(request, [obj])
            return redirect('.')
        return super().changeform_view(request, object_id, form_url, extra_context)

    def get_urls(self):
        urls = super(DjDynamicTemplateAdmin, self).get_urls()
        return [path('template-view/<int:template_id>/', template_view)] + urls

    # @staticmethod
    # def view_template(obj):
    #     return format_html(f"<a href='/admin/dj_dynamic_templates/djdynamictemplate/template-view/{obj.id}/' data-popup='yes' class='related-widget-wrapper-link'>Click Here to View Template</a>")

    @staticmethod
    def template_status(obj):
        if obj.template_is_active is False:
            return "Inactive"
        elif obj.category.is_directory_exists is False:
            return "Directory Does Not Exists"
        elif obj.is_file_exist is False:
            return "File Does Not Exists"
        else:
            with open(obj.file_path, 'r') as file:
                data = file.read().replace('\r', '')
            content = obj.content.replace('\r', '')
            if data == content:
                return "Synced"
            else:
                return "Not Synced"



from django.shortcuts import render

from .models import DjDynamicTemplate

# Create your views here.
def template_view(request, template_id):
    return render(request, DjDynamicTemplate.objects.get(id=template_id).file_path)
